'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var CoreApp = _interopDefault(require('fusion-core'));
var fusionReactAsync = require('fusion-react-async');
var ReactDOM = _interopDefault(require('react-dom'));
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

/* eslint-env node */

/* eslint-env browser */
var clientRender = (el => {
  const domElement = document.getElementById('root');
  return ReactDOM.hydrate ? ReactDOM.hydrate(el, domElement) : ReactDOM.render(el, domElement);
});

var Provider = {
  create: (name, BaseComponent = React.Component) => {
    class Provider extends BaseComponent {
      getChildContext() {
        return {
          [name]: this.props.service
        };
      }

      render() {
        return React.Children.only(this.props.children);
      }

    }

    Provider.childContextTypes = Object.assign({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.object.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';
    return Provider;
  }
};

var plugin = {
  create: (name, plugin, BaseComponent) => (...args) => {
    const p = plugin(...args);
    const middleware = p.__middleware__;
    const ProviderComponent = Provider.create(name, BaseComponent);

    p.__middleware__ = function (ctx, next) {
      if (ctx.element) {
        ctx.element = React.createElement(ProviderComponent, {
          service: this.of(ctx)
        }, ctx.element);
      }

      return middleware ? middleware.call(p, ctx, next) : next();
    };

    return p;
  }
};

var hoc = {
  create: (name, mapServiceToProps) => {
    if (!mapServiceToProps) {
      mapServiceToProps = service => ({
        [name]: service
      });
    }

    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.service = ctx[name];
        }

        render() {
          const props = Object.assign({}, this.props, mapServiceToProps(this.service));
          return React.createElement(Component, props);
        }

      }

      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.object.isRequired
      };
      return HOC;
    };
  }
};

/* eslint-env browser */
class App extends CoreApp {
  constructor(root, render) {
    super(root, el => {
      return fusionReactAsync.prepare(el).then(() => {
        if (render) {
          return render(el);
        }

        return clientRender(el);
      });
    });
    this.middleware(fusionReactAsync.PreparePlugin());
  }

}

exports['default'] = App;
exports.ProviderPlugin = plugin;
exports.ProvidedHOC = hoc;
exports.Provider = Provider;
//# sourceMappingURL=browser.es2017.cjs.js.map
