import CoreApp from 'fusion-core';
import { PreparePlugin, prepare } from 'fusion-react-async';
import { renderToString } from 'react-dom/server';
import React from 'react';
import PropTypes from 'prop-types';

/* eslint-env node */
var serverRender = (el => `<div id='root'>${renderToString(el)}</div>`);

/* eslint-env browser */

var Provider = {
  create: (name, BaseComponent = React.Component) => {
    class Provider extends BaseComponent {
      getChildContext() {
        return {
          [name]: this.props.service
        };
      }

      render() {
        return React.Children.only(this.props.children);
      }

    }

    Provider.childContextTypes = Object.assign({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.object.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';
    return Provider;
  }
};

var plugin = {
  create: (name, plugin, BaseComponent) => (...args) => {
    const p = plugin(...args);
    const middleware = p.__middleware__;
    const ProviderComponent = Provider.create(name, BaseComponent);

    p.__middleware__ = function (ctx, next) {
      if (ctx.element) {
        ctx.element = React.createElement(ProviderComponent, {
          service: this.of(ctx)
        }, ctx.element);
      }

      return middleware ? middleware.call(p, ctx, next) : next();
    };

    return p;
  }
};

var hoc = {
  create: (name, mapServiceToProps) => {
    if (!mapServiceToProps) {
      mapServiceToProps = service => ({
        [name]: service
      });
    }

    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.service = ctx[name];
        }

        render() {
          const props = Object.assign({}, this.props, mapServiceToProps(this.service));
          return React.createElement(Component, props);
        }

      }

      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.object.isRequired
      };
      return HOC;
    };
  }
};

/* eslint-env browser */
class App extends CoreApp {
  constructor(root, render) {
    super(root, el => {
      return prepare(el).then(() => {
        if (render) {
          return render(el);
        }

        return serverRender(el);
      });
    });
    this.middleware(PreparePlugin());
  }

}

export { plugin as ProviderPlugin, hoc as ProvidedHOC, Provider };
export default App;
//# sourceMappingURL=node.es.js.map
